<?php
/////////////////////////////////////////////////////////////////////////////////////////
//   Standard sanitization functions.
/////////////////////////////////////////////////////////////////////////////////////////
/**
 * Sanitizes a checkbox value.
 *
 * @param int|string|bool $input Input value to sanitize.
 * @return int|string Returns 1 if $input evaluates to 1, an empty string otherwise.
 */
function specialty_sanitize_checkbox( $input ) {
	if ( $input == 1 ) {
		return 1;
	}

	return '';
}


/**
 * Sanitizes a checkbox value. Value is passed by reference.
 *
 * Useful when sanitizing form checkboxes. Since browsers don't send any data when a checkbox
 * is not checked, specialty_sanitize_checkbox() throws an error.
 * specialty_sanitize_checkbox_ref() however evaluates &$input as null so no errors are thrown.
 *
 * @param int|string|bool &$input Input value to sanitize.
 * @return int|string Returns 1 if $input evaluates to 1, an empty string otherwise.
 */
function specialty_sanitize_checkbox_ref( &$input ) {
	if ( $input == 1 ) {
		return 1;
	}

	return '';
}


/**
 * Sanitizes the pagination method option.
 *
 * @param string $option Value to sanitize. Either 'numbers' or 'text'.
 * @return string
 */
function specialty_sanitize_pagination_method( $option ) {
	if( in_array( $option, array( 'numbers', 'text' ) ) ) {
		return $option;
	}

	return 'numbers';
}


/**
 * Sanitizes integer input while differentiating zero from empty string.
 *
 * @param int|string $input Input value to sanitize.
 * @return int|string Integer value, 0, or an empty string otherwise.
 */
function specialty_sanitize_intval_or_empty( $input ) {
	if ( false === $input || '' === $input || is_null( $input ) ) {
		return '';
	}

	if ( 0 === intval( $input ) ) {
		return 0;
	}

	return intval( $input );
}

/**
 * Sanitizes float input while differentiating zero from empty string.
 *
 * @param int|string $input Input value to sanitize.
 * @return int|string Integer value, 0, or an empty string otherwise.
 */
function specialty_sanitize_floatval_or_empty( $input ) {
	if ( false === $input || '' === $input || is_null( $input ) ) {
		return '';
	}

	if ( 0 === floatval( $input ) ) {
		return 0;
	}

	return floatval( $input );
}


/**
 * Returns a sanitized hex color code.
 *
 * @param string $str The color string to be sanitized.
 * @param bool $return_hash Whether to return the color code prepended by a hash.
 * @param string $return_fail The value to return on failure.
 * @return string A valid hex color code on success, an empty string on failure.
 */
function specialty_sanitize_hex_color( $str, $return_hash = true, $return_fail = '' ) {
	if( $str === false || empty( $str ) || $str == 'false' ) {
		return $return_fail;
	}

	// Allow keywords and predefined colors
	if ( in_array( $str, array( 'transparent', 'initial', 'inherit', 'black', 'silver', 'gray', 'grey', 'white', 'maroon', 'red', 'purple', 'fuchsia', 'green', 'lime', 'olive', 'yellow', 'navy', 'blue', 'teal', 'aqua', 'orange', 'aliceblue', 'antiquewhite', 'aquamarine', 'azure', 'beige', 'bisque', 'blanchedalmond', 'blueviolet', 'brown', 'burlywood', 'cadetblue', 'chartreuse', 'chocolate', 'coral', 'cornflowerblue', 'cornsilk', 'crimson', 'darkblue', 'darkcyan', 'darkgoldenrod', 'darkgray', 'darkgrey', 'darkgreen', 'darkkhaki', 'darkmagenta', 'darkolivegreen', 'darkorange', 'darkorchid', 'darkred', 'darksalmon', 'darkseagreen', 'darkslateblue', 'darkslategray', 'darkslategrey', 'darkturquoise', 'darkviolet', 'deeppink', 'deepskyblue', 'dimgray', 'dimgrey', 'dodgerblue', 'firebrick', 'floralwhite', 'forestgreen', 'gainsboro', 'ghostwhite', 'gold', 'goldenrod', 'greenyellow', 'grey', 'honeydew', 'hotpink', 'indianred', 'indigo', 'ivory', 'khaki', 'lavender', 'lavenderblush', 'lawngreen', 'lemonchiffon', 'lightblue', 'lightcoral', 'lightcyan', 'lightgoldenrodyellow', 'lightgray', 'lightgreen', 'lightgrey', 'lightpink', 'lightsalmon', 'lightseagreen', 'lightskyblue', 'lightslategray', 'lightslategrey', 'lightsteelblue', 'lightyellow', 'limegreen', 'linen', 'mediumaquamarine', 'mediumblue', 'mediumorchid', 'mediumpurple', 'mediumseagreen', 'mediumslateblue', 'mediumspringgreen', 'mediumturquoise', 'mediumvioletred', 'midnightblue', 'mintcream', 'mistyrose', 'moccasin', 'navajowhite', 'oldlace', 'olivedrab', 'orangered', 'orchid', 'palegoldenrod', 'palegreen', 'paleturquoise', 'palevioletred', 'papayawhip', 'peachpuff', 'peru', 'pink', 'plum', 'powderblue', 'rosybrown', 'royalblue', 'saddlebrown', 'salmon', 'sandybrown', 'seagreen', 'seashell', 'sienna', 'skyblue', 'slateblue', 'slategray', 'slategrey', 'snow', 'springgreen', 'steelblue', 'tan', 'thistle', 'tomato', 'turquoise', 'violet', 'wheat', 'whitesmoke', 'yellowgreen', 'rebeccapurple' ) ) ) {
		return $str;
	}

	// Include the hash if not there.
	// The regex below depends on in.
	if ( substr( $str, 0, 1 ) != '#' ) {
		$str = '#' . $str;
	}

	preg_match( '/(#)([0-9a-fA-F]{6})/', $str, $matches );

	if ( count( $matches ) == 3 ) {
		if ( $return_hash ) {
			return $matches[1] . $matches[2];
		} else {
			return $matches[2];
		}
	}

	return $return_fail;
}


/**
 * Return a list of allowed tags and attributes for a given context.
 *
 * @param string $context The context for which to retrieve tags.
 *                        Currently available contexts: guide
 * @return array List of allowed tags and their allowed attributes.
 */
function specialty_get_allowed_tags( $context = '' ) {
	$allowed = array(
		'a'       => array(
			'href'   => true,
			'title'  => true,
			'class'  => true,
			'target' => true,
			'rel'    => true,
		),
		'abbr'    => array( 'title' => true, ),
		'acronym' => array( 'title' => true, ),
		'b'       => array( 'class' => true, ),
		'br'      => array(),
		'code'    => array( 'class' => true, ),
		'em'      => array( 'class' => true, ),
		'i'       => array( 'class' => true, ),
		'img'     => array(
			'alt'    => true,
			'class'  => true,
			'src'    => true,
			'width'  => true,
			'height' => true,
		),
		'li'      => array( 'class' => true, ),
		'ol'      => array( 'class' => true, ),
		'p'       => array( 'class' => true, ),
		'pre'     => array( 'class' => true, ),
		'span'    => array( 'class' => true, ),
		'strong'  => array( 'class' => true, ),
		'ul'      => array( 'class' => true, ),
	);

	switch ( $context ) {
		case 'guide':
			unset( $allowed['p'] );
			break;
		default:
			break;
	}

	return apply_filters( 'specialty_get_allowed_tags', $allowed, $context );
}

/**
 * Sanitize user-provided CSS code, as recommended in https://make.wordpress.org/themes/2015/02/10/custom-css-boxes-in-themes/
 *
 * @param string $string The CSS code to sanitize.
 * @return string
 */
function specialty_sanitize_custom_css( $string ) {
	$string = wp_strip_all_tags( $string, false );

	return $string;
}

if ( ! function_exists( 'specialty_sanitize_footer_text' ) ) :
	function specialty_sanitize_footer_text( $text ) {
		return wp_kses( $text, specialty_get_allowed_tags( 'guide' ) );
	}
endif;

if ( ! function_exists( 'specialty_get_default_footer_text' ) ) :
	function specialty_get_default_footer_text( $position = 'left' ) {
		if ( 'right' === $position ) {
			if ( ! defined( 'SPECIALTY_WHITELABEL' ) || ! SPECIALTY_WHITELABEL ) {
				/* translators: %s is a url. */
				$text = sprintf( __( '<a href="%s">Specialty</a> &ndash; Job Board WordPress Theme', 'specialty' ),
					esc_url( 'https://www.cssigniter.com/ignite/themes/specialty/' )
				);
			} else {
				/* translators: %1$s is a url. %2$s the website's name. */
				$text = sprintf( __( '<a href="%1$s">%2$s</a>', 'specialty' ),
					esc_url( home_url( '/' ) ),
					get_bloginfo( 'name' )
				);
			}
		} else {
			if ( ! defined( 'SPECIALTY_WHITELABEL' ) || ! SPECIALTY_WHITELABEL ) {
				/* translators: %s is a url. */
				$text = sprintf( __( 'Powered by <a href="%s">CSSIgniter.com</a>', 'specialty' ),
					esc_url( 'https://www.cssigniter.com/' )
				);
			} else {
				/* translators: %s is a url. */
				$text = sprintf( __( 'Powered by <a href="%s">WordPress</a>', 'specialty' ),
					esc_url( 'https://wordpress.org/' )
				);
			}
		}

		return $text;
	}
endif;

if ( ! function_exists( 'specialty_sanitize_job_listing_layout_choices' ) ) :
	function specialty_sanitize_job_listing_layout_choices( $value ) {
		$choices = specialty_get_job_listing_layout_choices();
		if ( array_key_exists( $value, $choices ) ) {
			return $value;
		}

		return apply_filters( 'specialty_sanitize_listing_layout_choices_default', '' );
	}
endif;

if ( ! function_exists( 'specialty_sanitize_metabox_tab_hero' ) ) :
	function specialty_sanitize_metabox_tab_hero( $post_id ) {
		// Ignore phpcs issues. nonce validation happens inside specialty_can_save_meta(), from the caller of this function.
		// @codingStandardsIgnoreStart
		update_post_meta( $post_id, 'specialty_title', wp_kses( $_POST['specialty_title'], specialty_get_allowed_tags() ) );
		update_post_meta( $post_id, 'specialty_subtitle', wp_kses( $_POST['specialty_subtitle'], specialty_get_allowed_tags() ) );

		update_post_meta( $post_id, 'header_image_id', specialty_sanitize_intval_or_empty( $_POST['header_image_id'] ) );
		update_post_meta( $post_id, 'header_bg_color', specialty_sanitize_rgba_color( $_POST['header_bg_color'] ) );
		update_post_meta( $post_id, 'header_text_color', specialty_sanitize_hex_color( $_POST['header_text_color'] ) );
		update_post_meta( $post_id, 'header_image_repeat', specialty_sanitize_image_repeat( $_POST['header_image_repeat'] ) );
		update_post_meta( $post_id, 'header_image_position_x', specialty_sanitize_image_position_x( $_POST['header_image_position_x'] ) );
		update_post_meta( $post_id, 'header_image_position_y', specialty_sanitize_image_position_y( $_POST['header_image_position_y'] ) );
		update_post_meta( $post_id, 'header_image_attachment', specialty_sanitize_image_attachment( $_POST['header_image_attachment'] ) );
		update_post_meta( $post_id, 'header_image_cover', specialty_sanitize_checkbox_ref( $_POST['header_image_cover'] ) );
		// @codingStandardsIgnoreEnd
	}
endif;

if ( ! function_exists( 'specialty_get_image_repeat_choices' ) ) :
	function specialty_get_image_repeat_choices() {
		return apply_filters( 'specialty_image_repeat_choices', array(
			'no-repeat' => esc_html__( 'No repeat', 'specialty' ),
			'repeat'    => esc_html__( 'Tile', 'specialty' ),
			'repeat-x'  => esc_html__( 'Tile Horizontally', 'specialty' ),
			'repeat-y'  => esc_html__( 'Tile Vertically', 'specialty' ),
		) );
	}
endif;

if ( ! function_exists( 'specialty_sanitize_image_repeat' ) ) :
	function specialty_sanitize_image_repeat( $value ) {
		$choices = specialty_get_image_repeat_choices();
		if ( array_key_exists( $value, $choices ) ) {
			return $value;
		}

		return apply_filters( 'specialty_sanitize_image_repeat_default', 'no-repeat' );
	}
endif;

if ( ! function_exists( 'specialty_sanitize_image_position_x' ) ) :
	function specialty_sanitize_image_position_x( $value ) {
		$choices = specialty_get_image_position_x_choices();
		if ( array_key_exists( $value, $choices ) ) {
			return $value;
		}

		return apply_filters( 'specialty_sanitize_image_position_x_default', 'center' );
	}
endif;

if ( ! function_exists( 'specialty_get_image_position_x_choices' ) ) :
	function specialty_get_image_position_x_choices() {
		return apply_filters( 'specialty_image_position_x_choices', array(
			'left'   => esc_html__( 'Left', 'specialty' ),
			'center' => esc_html__( 'Center', 'specialty' ),
			'right'  => esc_html__( 'Right', 'specialty' ),
		) );
	}
endif;

if ( ! function_exists( 'specialty_sanitize_image_position_y' ) ) :
	function specialty_sanitize_image_position_y( $value ) {
		$choices = specialty_get_image_position_y_choices();
		if ( array_key_exists( $value, $choices ) ) {
			return $value;
		}

		return apply_filters( 'specialty_sanitize_image_position_y_default', 'center' );
	}
endif;

if ( ! function_exists( 'specialty_get_image_position_y_choices' ) ) :
	function specialty_get_image_position_y_choices() {
		return apply_filters( 'specialty_image_position_y_choices', array(
			'top'    => esc_html__( 'Top', 'specialty' ),
			'center' => esc_html__( 'Center', 'specialty' ),
			'bottom' => esc_html__( 'Bottom', 'specialty' ),
		) );
	}
endif;

if ( ! function_exists( 'specialty_sanitize_image_attachment' ) ) :
	function specialty_sanitize_image_attachment( $value ) {
		$choices = specialty_get_image_attachment_choices();
		if ( array_key_exists( $value, $choices ) ) {
			return $value;
		}

		return apply_filters( 'specialty_sanitize_image_attachment_default', 'scroll' );
	}
endif;

if ( ! function_exists( 'specialty_get_image_attachment_choices' ) ) :
	function specialty_get_image_attachment_choices() {
		return apply_filters( 'specialty_image_attachment_choices', array(
			'scroll' => esc_html__( 'Scroll', 'specialty' ),
			'fixed'  => esc_html__( 'Fixed', 'specialty' ),
		) );
	}
endif;

if ( ! function_exists( 'specialty_sanitize_rgba_color' ) ) :
	function specialty_sanitize_rgba_color( $str, $return_hash = true, $return_fail = '' ) {
		if ( false === $str || empty( $str ) || 'false' === $str ) {
			return $return_fail;
		}

		// Allow keywords and predefined colors
		if ( in_array( $str, array( 'transparent', 'initial', 'inherit', 'black', 'silver', 'gray', 'grey', 'white', 'maroon', 'red', 'purple', 'fuchsia', 'green', 'lime', 'olive', 'yellow', 'navy', 'blue', 'teal', 'aqua', 'orange', 'aliceblue', 'antiquewhite', 'aquamarine', 'azure', 'beige', 'bisque', 'blanchedalmond', 'blueviolet', 'brown', 'burlywood', 'cadetblue', 'chartreuse', 'chocolate', 'coral', 'cornflowerblue', 'cornsilk', 'crimson', 'darkblue', 'darkcyan', 'darkgoldenrod', 'darkgray', 'darkgrey', 'darkgreen', 'darkkhaki', 'darkmagenta', 'darkolivegreen', 'darkorange', 'darkorchid', 'darkred', 'darksalmon', 'darkseagreen', 'darkslateblue', 'darkslategray', 'darkslategrey', 'darkturquoise', 'darkviolet', 'deeppink', 'deepskyblue', 'dimgray', 'dimgrey', 'dodgerblue', 'firebrick', 'floralwhite', 'forestgreen', 'gainsboro', 'ghostwhite', 'gold', 'goldenrod', 'greenyellow', 'grey', 'honeydew', 'hotpink', 'indianred', 'indigo', 'ivory', 'khaki', 'lavender', 'lavenderblush', 'lawngreen', 'lemonchiffon', 'lightblue', 'lightcoral', 'lightcyan', 'lightgoldenrodyellow', 'lightgray', 'lightgreen', 'lightgrey', 'lightpink', 'lightsalmon', 'lightseagreen', 'lightskyblue', 'lightslategray', 'lightslategrey', 'lightsteelblue', 'lightyellow', 'limegreen', 'linen', 'mediumaquamarine', 'mediumblue', 'mediumorchid', 'mediumpurple', 'mediumseagreen', 'mediumslateblue', 'mediumspringgreen', 'mediumturquoise', 'mediumvioletred', 'midnightblue', 'mintcream', 'mistyrose', 'moccasin', 'navajowhite', 'oldlace', 'olivedrab', 'orangered', 'orchid', 'palegoldenrod', 'palegreen', 'paleturquoise', 'palevioletred', 'papayawhip', 'peachpuff', 'peru', 'pink', 'plum', 'powderblue', 'rosybrown', 'royalblue', 'saddlebrown', 'salmon', 'sandybrown', 'seagreen', 'seashell', 'sienna', 'skyblue', 'slateblue', 'slategray', 'slategrey', 'snow', 'springgreen', 'steelblue', 'tan', 'thistle', 'tomato', 'turquoise', 'violet', 'wheat', 'whitesmoke', 'yellowgreen', 'rebeccapurple' ), true ) ) {
			return $str;
		}

		preg_match( '/rgba\(\s*(\d{1,3}\.?\d*\%?)\s*,\s*(\d{1,3}\.?\d*\%?)\s*,\s*(\d{1,3}\.?\d*\%?)\s*,\s*(\d{1}\.?\d*\%?)\s*\)/', $str, $rgba_matches );
		if ( ! empty( $rgba_matches ) && 5 === count( $rgba_matches ) ) {
			for ( $i = 1; $i < 4; $i++ ) {
				if ( strpos( $rgba_matches[ $i ], '%' ) !== false ) {
					$rgba_matches[ $i ] = specialty_sanitize_0_100_percent( $rgba_matches[ $i ] );
				} else {
					$rgba_matches[ $i ] = specialty_sanitize_0_255( $rgba_matches[ $i ] );
				}
			}
			$rgba_matches[4] = specialty_sanitize_0_1_opacity( $rgba_matches[ $i ] );
			return sprintf( 'rgba(%s, %s, %s, %s)', $rgba_matches[1], $rgba_matches[2], $rgba_matches[3], $rgba_matches[4] );
		}

		// Not a color function either. Let's see if it's a hex color.

		// Include the hash if not there.
		// The regex below depends on in.
		if ( substr( $str, 0, 1 ) !== '#' ) {
			$str = '#' . $str;
		}

		preg_match( '/(#)([0-9a-fA-F]{6})/', $str, $matches );

		if ( 3 === count( $matches ) ) {
			if ( $return_hash ) {
				return $matches[1] . $matches[2];
			} else {
				return $matches[2];
			}
		}

		return $return_fail;
	}
endif;

if ( ! function_exists( 'specialty_sanitize_0_100_percent' ) ) :
	function specialty_sanitize_0_100_percent( $val ) {
		$val = str_replace( '%', '', $val );
		if ( floatval( $val ) > 100 ) {
			$val = 100;
		} elseif ( floatval( $val ) < 0 ) {
			$val = 0;
		}

		return floatval( $val ) . '%';
	}
endif;

if ( ! function_exists( 'specialty_sanitize_0_255' ) ) :
	function specialty_sanitize_0_255( $val ) {
		if ( intval( $val ) > 255 ) {
			$val = 255;
		} elseif ( intval( $val ) < 0 ) {
			$val = 0;
		}

		return intval( $val );
	}
endif;

if ( ! function_exists( 'specialty_sanitize_0_1_opacity' ) ) :
	function specialty_sanitize_0_1_opacity( $val ) {
		if ( floatval( $val ) > 1 ) {
			$val = 1;
		} elseif ( floatval( $val ) < 0 ) {
			$val = 0;
		}

		return floatval( $val );
	}
endif;